from datetime import datetime
from enum import Enum
from peewee import (
    BooleanField,
    CharField,
    DateTimeField,
    ForeignKeyField,
    Model,
    SqliteDatabase,
    TextField,
)

db = SqliteDatabase(
    "tasks.db",
    pragmas={
        "journal_mode": "wal",
        "synchronous": "normal",
        "foreign_keys": 1,
    },
)


class TaskStatus(Enum):
    # order is used for progression in toggle
    ZERO = "zero"
    WIP = "wip"
    BLOCKED = "blocked"
    DONE = "done"


class BaseModel(Model):
    class Meta:
        database = db


class Category(BaseModel):
    name = CharField(unique=True)

    def __str__(self):
        return self.name


class Task(BaseModel):
    # TODO: rename name
    text = TextField()
    status = CharField(
        choices=[(status.value, status.name) for status in TaskStatus],
        default=TaskStatus.ZERO.value,
    )
    due = DateTimeField(null=True)
    category = ForeignKeyField(Category, backref="tasks", null=True)
    type = CharField()
    created_at = DateTimeField(default=datetime.now)
    updated_at = DateTimeField(default=datetime.now)
    deleted = BooleanField(default=False)

    def save(self, *args, **kwargs):
        self.updated_at = datetime.now()
        return super(Task, self).save(*args, **kwargs)


class SavedSearch(BaseModel):
    name = CharField(unique=True)
    filters = CharField()
    sort_string = CharField()

    def __str__(self):
        return self.name


def initialize_db():
    db.connect()
    db.create_tables([Category, Task, SavedSearch])
    if not Category.select().exists():
        Category.create(name="default")
    db.close()


if __name__ == "__main__":
    initialize_db()
